# coding: utf-8
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.request import Request
from rest_framework.response import Response

from before_request import before_request
from config.config import name_list
from inspect_report.models import CustomProfile
import logging

logger = logging.getLogger(__name__)


class ProfileApi(viewsets.ViewSet):
    authentication_classes = ()
    permission_classes = ()

    @action(['get'], detail=False)
    @before_request
    def labels(self, req: Request):
        page = req.GET.get('page', '1')
        page_size = req.GET.get('page_size', '10')
        region = req.GET.get('region', '')
        username = req.data.get('username', '')
        if username in name_list:
            logger.info('[profile_labels]获取的username是: %s', username)
            region = username
        condition = {'custom_labels__isnull': False}
        if region and region != '安徽picc':
            condition['region'] = region
        profiles = CustomProfile.objects.filter(**condition).order_by('-call_recent').values('phone_number', 'region',
                                                                                             'call_recent',
                                                                                             'call_count',
                                                                                             'custom_labels')
        paginator = Paginator(profiles, page_size)
        total_count = paginator.count
        try:
            objects = paginator.page(page)
        except PageNotAnInteger:
            objects = paginator.page(1)
        except EmptyPage:
            objects = paginator.page(paginator.num_pages)
        return Response({'code': 0, 'msg': '查询成功', 'total': total_count, 'data': objects.object_list})
