# -*- coding: utf-8 -*-
import logging
from rest_framework.permissions import BasePermission
from rest_framework.authentication import exceptions
from config.config import CUSTOM_PROFILE_TOKEN

log = logging.getLogger(__name__)


class ValidateToken(BasePermission):
    """ token 正确才可以调用 """
    def has_permission(self, request, view):
        try:
            http_auth = request.META['HTTP_AUTHORIZATION']
            if http_auth:
                token = http_auth.split(' ')[1]
                if http_auth.split(' ')[0] != 'Token':
                    log.error('token 认证异常 | HTTP_AUTHORIZATION 中缺少 Token')
                    raise exceptions.AuthenticationFailed(detail={'code': -5,
                                                                  'msg': 'token认证异常,HTTP_AUTHORIZATION 中缺少 Token'})
                if token == CUSTOM_PROFILE_TOKEN:
                    log.info('token 认证成功')
                    return True
                else:
                    raise exceptions.AuthenticationFailed(detail={'code': -5, 'msg': 'token 认证异常'})
        except KeyError as ke:
            log.error('token 认证异常 | header中缺少HTTP_WXAUTH | 异常信息 : KeyError : %s', str(ke))
            raise exceptions.AuthenticationFailed(detail={'code': -5, 'msg': 'token 认证异常'})
        except IndexError as ie:
            log.error('token 认证异常 | HTTP_WXAUTH 中值错误 : %s | 异常信息 : IndexError : %s',
                      request.META['HTTP_WXAUTH'], str(ie))
            raise exceptions.AuthenticationFailed(detail={'code': -5, 'msg': 'token 认证异常'})
        except Exception as e:
            log.error('token 认证异常 : %s', str(e))
            raise exceptions.AuthenticationFailed(detail={'code': -5, 'msg': 'token 认证异常'})
        raise exceptions.AuthenticationFailed(detail={'code': -5, 'msg': 'token 认证异常'})
